# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo

import mcpi

from .house import House
from .thing import Thing


class Building(Thing):

    floors = 10
    width = 10
    house_mirror = False

    def create(self):

        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        house_pos = mcpi.vec3.Vec3(init_x, init_y, init_z)
        init_height = init_y

        for i in range(0, self.floors):
            house = House(house_pos, self)
            self.add_child(house)
            house_pos.y = house.height * i + init_height
            house.width = self.width
            house.block = self.block
            house.mirror = self.house_mirror
            house.create()
            self._end_position = house.end_position



# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo

import mcpi.block
from mcpi.vec3 import Vec3

from .thing import Thing
from .world import World


class Line(Thing):

    width = 3
    length = 10

    def create(self):
        end_x = self.position.x + self.width - 1
        end_y = self.position.y - 1
        end_z = self.position.z + self.length - 1

        # Find the type of land block destroyed with the line
        self._block_empty = \
            mcpi.block.Block(World.renderer.get_block(Vec3(self.position.x, self.position.y-1, self.position.z)), 0)

        self.set_blocks(Vec3(self.position.x, self.position.y-1, self.position.z),
                        Vec3(end_x, end_y, end_z),
                        self.block.id)

        self._end_position = Vec3(end_x, end_y + 1, end_z)



# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo


class Renderer:
    """ Base class for all McThings Renderers """

    """
    Base class for all McThings Renderers
    
    A renderer reads the blocks data from mcthings.core.BlocksMemory and render it
    with a specific engine. For example, Raspberry PI uses the Python API to do it.
    """

    def render(self, blocks_memory):
        """
        Render the blocks included in the memory_chunk at position in the world

        :param blocks_memory: memory with the blocks to be rendered
        :return:
        """

    def post_to_chat(self, message):
        """
        Send a message to the chat in the renderer it it exists
        :param message:
        :return:
        """

    def get_block(self, position):
        """
        Get the rendered block at the given position
        :param position:
        :return:
        """

    def get_blocks(self, init_pos, end_pos):
        """
        Get the rendered cuboid at init_pos and end_pos
        :param init_pos:
        :param end_pos:
        :return:
        """


# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo

from mcpi.vec3 import Vec3

from .thing import Thing


class Blocks(Thing):
    width = 3  # x
    height = 2  # y
    length = 4  # z

    def create(self):
        p = self.position
        self._end_position = Vec3(p.x + self.width - 1, p.y + self.height - 1, p.z + self.length - 1)
        # self.set_blocks(Vec3(p.x, p.y, p.z), self._end_position, self.block.id)
        self.set_blocks(self._end_position, Vec3(p.x, p.y, p.z), self.block.id)



# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo

import mcpi

from mcpi.vec3 import Vec3

from .thing import Thing


class House(Thing):

    height = 3
    width = 5
    length = 5
    wall_width = 1
    door_size = 1
    mirror = False

    def create(self):

        init_x = self.position.x
        init_y = self.position.y
        init_z = self.position.z

        end_x = init_x + self.length
        if self.mirror:
            end_x = init_x
            init_x = init_x - self.length
        end_y = init_y + self.height
        end_z = init_z + self.width

        self.set_blocks(
            Vec3(init_x, init_y, init_z),
            Vec3(end_x, end_y, end_z),
            self.block.id)

        # Fill the cube with air so it becomes a kind of house
        init_x_empty = init_x + self.wall_width
        end_x_empty = end_x - self.wall_width
        self.set_blocks(
            Vec3(init_x_empty, init_y, init_z + self.wall_width),
            Vec3(end_x_empty,
                 end_y - self.wall_width,
                 end_z - self.wall_width),
            mcpi.block.AIR.id)

        init_door_x = init_x
        if self.mirror:
            init_door_x = end_x
        init_door = Vec3(init_door_x, init_y, init_z + self.wall_width)
        end_door = Vec3(init_door_x + self.wall_width, init_y + self.door_size + 1, init_z + self.wall_width + self.door_size)
        self.set_blocks(init_door, end_door, mcpi.block.AIR.id)

        self._end_position = Vec3(end_x, end_y, end_z)



# Licensed under the terms of http://www.apache.org/licenses/LICENSE-2.0
# Author (©): Alvaro del Castillo

from mcpi.vec3 import Vec3
import mcpi.block

from .thing import Thing


class Collage(Thing):
    width = 3  # x
    height = 2  # y
    length = 4  # z
    change_blocks = [mcpi.block.BEDROCK, mcpi.block.SAND, mcpi.block.GOLD_BLOCK, mcpi.block.IRON_BLOCK]

    def create(self):
        p = self.position
        count = 0
        for y in range(0, self.height):
            for z in range(0, self.length):
                for x in range(0, self.width):
                    block = self.block
                    if self.block != self._block_empty:
                        block = self.change_blocks[count % len(self.change_blocks)]
                    self.set_block(Vec3(p.x + x, p.y + y, p.z + z), block.id)
                    count += 1

        self._end_position = Vec3(p.x + self.width - 1, p.y + self.height - 1, p.z + self.length - 1)
